<?php

declare(strict_types=1);

namespace app\models;

use yii;

use mirzaev\yii2\arangodb\ActiveRecord;

use Exception;

/**
 * Документ
 */
abstract class Document extends ActiveRecord
{
    /**
     * Имя коллекции
     */
    public static function collectionName(): string
    {
        return throw new Exception('Не инициализировано название коллекции');
    }

    /**
     * Свойства
     */
    public function attributes(): array
    {
        return [
            '_key',
            'jrnl'
        ];
    }

    /**
     * Метки свойств
     */
    public function attributeLabels(): array
    {
        return [
            '_key' => 'Ключ',
            'jrnl' => 'Журнал'
        ];
    }


    /**
     * Правила
     *
     * @todo Добавить проверку существования аккаунта
     */
    public function rules(): array
    {
        return [];
    }

    /**
     * Перед сохранением
     *
     * @todo Подождать обновление от ебаного Yii2 и добавить
     * проверку типов передаваемых параметров
     */
    public function beforeSave($data): bool
    {
        if (parent::beforeSave($data)) {
            if ($this->isNewRecord) {
                // Запись в журнал
                $this->jrnl = [[
                    'date' => time(),
                    'account' => yii::$app->user->id,
                    'action' => 'create'
                ]];
            }

            return true;
        }

        return false;
    }

    /**
     * Журнал
     *
     * Записывает данные в журнал
     *
     * @param string $action
     *
     * @return int|bool Время записанное в журнале или false, если не удалось записать
     */
    public function journal(string $action = 'update', array ...$data): int|bool
    {
        // Инициализация
        is_array($this->jrnl) or $this->jrnl = [];

        // Генерация
        $this->jrnl = array_merge(
            $this->jrnl,
            [array_merge(
                [
                    'date' => $time = time(),
                    'account' => yii::$app->user->id,
                    'action' => $action
                ],
                ...$data
            )]
        );

        // Запись и возврат
        return $this->save() ? $time : false;
    }

    /**
     * Чтение идентификатора
     */
    public function readId(): ?string
    {
        return isset($this->_key) && static::collectionName() ? static::collectionName() . '/' . $this->_key : null;
    }

    /**
     * Поиск по идентификатору
     */
    public static function searchById(string $_id): ?static
    {
        return static::findOne(['_id' => $_id]);
    }

    public static function readLast(): ?static
    {
        return static::find()->orderBy(['DESC'])->one();
    }

    /**
     * Чтение всех записей
     */
    public static function readAll(): array
    {
        return static::find()->all();
    }

    /**
     * Чтение количества записей
     */
    public static function readAmount(): int
    {
        return static::find()->count();
    }

    /**
     * Проверка на то, что в свойство передан массив
     */
    public function arrayValidator(string $attribute, array $params = null): bool
    {
        if (is_array($this->$attribute)) {
            return true;
        }

        return false;
    }
}
